/**
dhcp.h
 */

#ifndef _DHCP_H_
#define _DHCP_H_

#include "w5100.h"
/* DHCP state machine. */
#define	STATE_DHCP_DISCOVER	    1
#define	STATE_DHCP_REQUEST	    2
#define	STATE_DHCP_LEASED	    3
#define	STATE_DHCP_REREQUEST	4
#define	STATE_DHCP_RELEASE	    5

#define	MAX_DHCP_RETRY		    3
#define	DHCP_WAIT_TIME		    5

#define DHCP_FLAGSBROADCAST	0x8000

/* UDP port numbers for DHCP */
#define	DHCP_SERVER_PORT	0x0043	/* from server to client */
#define DHCP_CLIENT_PORT	0x0044	/* from client to server */

/* DHCP message OP code */
#define DHCP_BOOTREQUEST	1
#define DHCP_BOOTREPLY		2

/* DHCP message type */
#define	DHCP_DISCOVER		1
#define DHCP_OFFER		    2
#define	DHCP_REQUEST		3
#define	DHCP_DECLINE		4
#define	DHCP_ACK		    5
#define DHCP_NAK		    6
#define	DHCP_RELEASE		7
#define DHCP_INFORM 		8

/* DHCP RETRANSMISSION TIMEOUT (microseconds) */
#define DHCP_INITIAL_RTO	( 4*1000000)
#define DHCP_MAX_RTO		(64*1000000)

#define DHCP_HTYPE10MB		1
#define DHCP_HTYPE100MB		2

#define DHCP_HLENETHERNET	6
#define DHCP_HOPS		0
#define DHCP_SECS		0

#define MAGIC_COOKIE		0x63825363

#define DEFAULT_LEASETIME	0xffffffff	/**< infinite lease time */


typedef union _un_l2cval {
    unsigned int  lVal;
    unsigned char  cVal[4];
}un_l2cval;

typedef union _un_i2cval {
    unsigned short   iVal;
    unsigned char  cVal[2];
}un_i2cval;



#define TX_RX_MAX_BUF_SIZE  2048
#define TX_BUF  0x1100
#define RX_BUF  (TX_BUF+TX_RX_MAX_BUF_SIZE)

/** 
 *opciones de configuracion del paquete DHCP	
 * DHCP option and value (cf. RFC1533) 
 */
enum
{
    pad_option		            =	0,
	subnet_mask	    	        =	1,
	timer_offset		        =	2,
	routers_on_subnet		    =	3,
	time_Server		            =	4,
	name_Server		            =	5,
	dns			                =	6,
	log_server	    	        =	7,
	cookie_server		        =	8,
	lpr_server		            =	9,
	impress_server		        =	10,
	resource_location_server	=	11,
	host_name		            =	12,
	boot_file_size		        =	13,
	merit_dump_file		        =	14,
	domain_name		            =	15,
	swap_server		            =	16,
	root_path		            =	17,
	extentions_path		        =	18,
	IPforwarding		        =	19,
	non_local_source_routing    =	20,
	policy_filter		        =	21,
	max_dgram_reasm_size        =	22,
	default_IPTTL		        =	23,
	path_MTU_aging_Timeout      =	24,
	path_MTU_plateau_table      =	25,
	ifMTU			            =	26,
	all_subnets_local	        =	27,
	broadcast_addr		        =	28,
	perform_mask_discovery	    =	29,
	mask_supplier		        =	30,
	perform_router_discovery    =	31,
	router_solicitation_addr    =	32,
	static_route	            =	33,
	trailer_encapsulation	    =	34,
	arp_cache_timeout	        =	35,
	ethernet_encapsulation	    =	36,
	tcp_default_TTL		        =	37,
	tcp_keepalive_interval	    =	38,
	tcp_keepalive_garbage	    =	39,
	nis_domain_name		        =	40,
	nis_servers		            =	41,
	ntp_servers		            =	42,
	vendor_specific_info	    =	43,
	netBIOS_name_server         =	44,
	netBIOS_dgram_dist_server   =	45,
	netBIOS_node_type           =	46,
	netBIOS_scope		        =	47,
	xfont_server		        =	48,
	xdisplay_manager		    =	49,
	dhcp_requested_IPaddr	    =	50,
	dhcp_IPaddr_lease_time	    =	51,
	dhcp_option_overload	    =	52,
	dhcp_message_type		    =	53,
	dhcp_server_identifier	    =	54,
	dhcp_param_request     	    =	55,
	dhcp_msg       		        =	56,
	dhcp_max_msg_size	        =	57,
	dhcp_T1_value	            =	58,
	dhcp_T2_value	            =	59,
	dhcp_class_identifier       =	60,
	dhcp_client_identifier	    =	61,
	end_option		            =	255   

};


/** 
 * @brief		for the DHCP message
*/
typedef struct _RIP_MSG
{
	unsigned char	op;    /// byte opcode 1= bootrequest, 2=BOOTREPlY
	unsigned char	htype; /// byte tipo de dir hw "1"= 10Mb eth
	unsigned char	hlen;  /// byte long del la dir hw "6" para 10Mb eth
	unsigned char	hops;  /// byte clientes envian a zero usuado para agent relays
	unsigned int	xid;   /// 4 bytes  transaction id # aleatorio escojido por el cliente y el serv para asociar los msj
    unsigned short  secs;   /// 2 bytes segundos que han parse desde q el cliente empezo el requerimiento de ip
	unsigned short	flags;  ///2 bytes  flags
	unsigned char	ciaddr[4]; /// client ip addr si la tiene
	unsigned char	yiaddr[4]; /// your ip addr respuesta del serv al cliente
	unsigned char	siaddr[4]; /// server ip addr  enviada por el srv
	unsigned char	giaddr[4]; /// agente de retransmision 
	unsigned char	chaddr[16]; /// client hw addr
	unsigned char	sname[64]; /// optional server host name
	unsigned char	file[128]; /// boot file name
	unsigned char	OPT[312]; /// campo de parametros opcional
}RIP_MSG;
 
 
#define MAX_DHCP_OPT	16


//extern void send_DHCP_DISCOVER(SOCKET s);
  

// void send_DHCP_RELEASE_DECLINE(SOCKET s, char msgtype);

extern void init_dhcp_client(SOCKET s);/////, void (*ip_update)(void),void (*ip_conflict)(void));	// Initialize the DHCP client
/*extern unsigned short*/ unsigned int init_dhcpc_ch(SOCKET s);								// Initialize the socket for DHCP client
extern unsigned int getIP_DHCPS(void);								// Get the network configuration from the DHCP server
extern unsigned char check_DHCP_state(SOCKET s);							// Check the DHCP state
//unsigned char parseDHCPMSG(SOCKET s, unsigned short length); /* Receive the message from DHCP server and parse it. */
void get_myIP(unsigned char *ip1, unsigned char *ip2, unsigned char *ip3, unsigned char *ip4);       

#endif	/* _DHCP_H_ */
